# -*- coding: utf-8 -*-

import logging
from openerp import models, fields, api, _
from openerp.exceptions import Warning

_logger = logging.getLogger(__name__)

class ProductPricelistPorcent(models.Model):
    _name = 'product.pricelist.porcent'

    minimum_percent = fields.Float(digits=(5, 2))
    maximum_percent = fields.Float(digits=(5, 2))

    price_minimum = fields.Float(string="Price Minimo")
    price_maximum = fields.Float(string="Price Maximo")
    price_coste = fields.Float(string="Price coste")

    product_id  = fields.Many2one('product.product', 'Product Variant')
    product_tmpl_id = fields.Many2one('product.template', 'Product')
    pricelist_id = fields.Many2one('product.pricelist', 'Pricelist')

''' PRODUCT TEMPLATE '''
class ProductTemplatePricelist(models.Model):
    _inherit = 'product.template'

    pricelist_procent_id = fields.One2many('product.pricelist.porcent', 'product_tmpl_id', string='pricelist porcent',)

''' PRODUCT PRODUCT '''
class ProductProductPricelist(models.Model):
    _inherit = 'product.product'

    pricelist_procent_id = fields.One2many('product.pricelist.porcent', 'product_id', string='pricelist porcent',)

''' PRODUCT PRICELIST '''
class ProductPricelistUtility(models.Model):
    _inherit = 'product.pricelist'

    product_price_minimum = fields.Float(compute="_get_product_price_minimum",string="Price Minimo")
    pricelist_procent_id = fields.One2many('product.pricelist.porcent', 'pricelist_id', string='pricelist porcent',)

    @api.one
    def _get_product_price_minimum(self):
        product_id = self._get_product_id()
        if product_id:
            percent = self.env['product.pricelist.porcent'].search([('product_id', '=', product_id),('pricelist_id', '=', self.id)])
            self.product_price_minimum = percent.price_minimum or self.price_get(product_id, 1).get(self.id, 0.0)

    @api.model
    def get_pricelist_sale(self, productId):
        product = self.env['product.product'].browse(productId)

        standard_price = 0.0
        if (product):
            standard_price = product.standard_price or 0.0

        priceList = self.env['product.pricelist'].search([('type', '=', 'sale'),('active', '=', True),('show_on_products', '=', True)])
        if (not priceList):
            return False

        listPrice = []
        for list in priceList:

            percent = self.env['product.pricelist.porcent'].search([('product_id', '=', productId),('pricelist_id', '=', list.id)])
            listPrice.append({
                'id': list.id,
                'name': list.name,
                'showOnProducts': list.show_on_products,
                'productPrice': list.product_price,
                'productPriceManual': list.product_price_manual,
                'productId': list.product_id,
                'productPriceMinimum': list.product_price_minimum,
                'standardPrice': standard_price,
                'minimumPercent': percent.minimum_percent or 0.0,
                'maximumPercent': percent.maximum_percent or 0.0,
                'priceMinimum': percent.price_minimum or 0.0,
                'priceMaximum': percent.price_maximum or 0.0,
                'currency': {
    				'id': list.currency_id.id,
    				'symbol': list.currency_id.symbol,
    				'decimalSeparator': list.currency_id.decimal_separator,
    				'decimalPlaces': list.currency_id.decimal_places,
    				'thousandsSeparator': list.currency_id.thousands_separator,
        		},
            })

        if (not listPrice):
            return False

        return listPrice

    @api.model
    def calculate_priceList_percent(self, coste, percent):
        decimal_precision = self.env['decimal.precision'].precision_get('Account')
        coste = float(coste)
        percent = float(percent)
        amount = round(coste * (1 + (percent/100)), decimal_precision)
        return amount or 0

    @api.model
    def update_priceList_percent(self, values):
        for line in values:
            ''' product.product '''
            product = self.env['product.product'].browse(line['product_id'])
            if (not product):
                return {'state': False, 'message': "Error en obtener el producto"}

            ''' product.pricelist '''
            pricelist = self.env['product.pricelist'].browse(line['list_id'])
            if (not pricelist):
                return {'state': False, 'message': "Error en obtener la Lista de precio"}

            ''' product.pricelist.porcent '''
            pricelistPercent = self.env['product.pricelist.porcent'].search([('product_id', '=', product.id), ('pricelist_id', '=', pricelist.id)])
            if (pricelistPercent):
                pricelistPercent.unlink()

            percent = {
                'minimum_percent': line['minimum_percent'],
                'maximum_percent': line['maximum_percent'],
                'price_minimum': line['price_minimum'],
                'price_maximum': line['price_maximum'],
                'price_coste': line['standard_price'],
                'product_id':product.id ,
                'product_tmpl_id': product.product_tmpl_id.id,
                'pricelist_id':pricelist.id ,
            }
            pricelistPercent = self.env['product.pricelist.porcent'].create(percent)
            newPrice = pricelist.price_set(product.product_tmpl_id, line['price_maximum'], product.id)
            product.write({'lst_price': line['price_maximum']})

        return {
            'state': False,
            'message': "Lista de precio Actualizada"
        }

    @api.multi
    def price_set(self, product_template, new_price, product=False):
        """
        Set the price of the product in current pricelist
        if different from price through  pricerules
        :param product_template: product_template object
        :param product: product_id change price
        :param new_price: new price
        :return:
        """
        if new_price:
            version = self.get_active_pricelist_version()
            if not version:
                return False
            items = self.env['product.pricelist.item'].search(
                    [
                        ('product_tmpl_id','=', product_template.id),
                        ('price_version_id', '=', version.id),
                        ('product_id', '=', product)
                    ]
            )
            product_price_type_ids = self.env['product.price.type'].search(
                    [
                        ('field', '=', 'list_price')
                    ]
            )
            if not items:
                self.env['product.pricelist.item'].create(
                        {
                            'base': product_price_type_ids and product_price_type_ids[0].id,
                            'sequence': 1,
                            'name': product_template.name,
                            'product_tmpl_id': product_template.id,
                            'price_version_id': version.id,
                            'price_surcharge': new_price,
                            'price_discount': -1,
                            'product_id': product,
                        }
                )
            else:
                for item in items:
                    item.write(
                            {
                                'base': product_price_type_ids and product_price_type_ids[0].id,
                                'sequence': 1,
                                'name': product_template.name,
                                'product_tmpl_id': product_template.id,
                                'price_version_id': version.id,
                                'price_surcharge': new_price,
                                'product_id': product,
                                'price_discount': -1
                            }
                    )
            return True


class SaleOrderPricelist(models.Model):
    _inherit = 'sale.order'

    @api.multi
    def action_button_confirm(self):
        message1 = ""
        for xsale in self.order_line:
            listPercent = self.env['product.pricelist.porcent'].search([('product_id', '=', xsale.product_id.id), ('pricelist_id', '=', self.pricelist_id.id)])
            if (listPercent):
                if (xsale.price_unit < listPercent.price_minimum):
                    message1 += "\n Producto (%s)    Precio de Venta (%s)    Precio de Minimo (%s)" %(xsale.name.encode("utf-8") ,xsale.price_unit, listPercent.price_minimum)

        message = "¡Precio de venta no permitido!\n¡El monto ingresado es menor  al precio mínimo permitido!\n"
        message += message1
        if message1:
            raise Warning(_(message))

        return super(SaleOrderPricelist, self).action_button_confirm()

''' Sale Order LIne '''
class SaleOrderLinePricelist(models.Model):
    _inherit = 'sale.order.line'

    @api.onchange('price_unit')
    def _change_pricelist_validate(self):
        if (not self.product_id):
            return

        if (not self.order_id.pricelist_id):
            return

        listPercent = self.env['product.pricelist.porcent'].search([('product_id', '=', self.product_id.id), ('pricelist_id', '=', self.order_id.pricelist_id.id)])
        if (not listPercent):
            return

        if (self.price_unit < listPercent.price_minimum):
            price = self.price_unit
            msg = 'El valor ingresado es menor al precio mínimo permito.\
                    Valor Ingresado %s precio  mínimo %s"' %(price,listPercent.price_minimum)
            raise Warning(_(msg))
